import numpy as np
from .. import algebra
from numba import jit

@jit(nopython=True)
def maximum_wannier_gap(m):
    """Return the maximum wannier gap"""
    x = m[0]
    # find the position of the maximum gap at every t
    fermis = x*0. # maximum gap
    for it in range(len(x)): # loop over times
        dmax = -1 # initialize
        maxgap = -1.0 # maximum gap
        gapangle = ((m[1][it]+0.5)%1)*np.pi # initialize
        for i in range(1,len(m)):
          for j in range(i+1,len(m)):
            for ipi in [0.,1.]:
              ip = np.exp(1j*m[i][it]) # center of wave i
              jp = np.exp(1j*m[j][it]) # center of wave j
              angle = np.angle(ip+jp)+np.pi*ipi # get the angle
              dp = np.exp(1j*angle) # now obtain this middle point gap
              mindis = 4.0 # calculate minimum distance
              for k in range(1,len(m)): # loop over centers
                kp = np.exp(1j*m[k][it]) # center of wave k
                dis = np.abs(dp-kp) # distance between the two points
                if dis<mindis: mindis = dis+0. # update minimum distance
              if mindis>maxgap: # if found a bigger gap
                maxgap = mindis+0. # maximum distance
                gapangle = np.angle(dp) # update of found bigger gap
        fermis[it] = gapangle
    return fermis


from .overlap import uij


def smooth_gauge(w1,w2):
    """Regauge wavefunctions w2 so that they are
    adibatically connected to w1"""
    m = uij(w1,w2) # matrix of wavefunctions
    U, s, V = np.linalg.svd(m, full_matrices=True) # sing val decomp
    R = algebra.dagger(U@V) # rotation matrix
    return smooth_rotation_jit(w2,R)

@jit(nopython=True)
def smooth_rotation_jit(w2,R):
    wnew = w2.copy()*0j # initialize
    wold = w2.copy() # old waves
    for ii in range(R.shape[0]):
      for jj in range(R.shape[0]):
        wnew[ii] += R[jj,ii]*wold[jj]
    return wnew









# function to modify the Hamiltonian matrices according to criteria


def remove_hopping(self,f):
    """Remove hoppings to site according to criteria from the geometry"""
    g = self.geometry
    def fm(m): # function to modify matrices
        for i in range(len(g.r)): # loop over sites
            if f(g.r[i]): # if this site is removed
                if self.has_spin and not self.has_eh:
                    for j in range(m.shape[0]):
                        m[2*i,j] = 0.
                        m[2*i+1,j] = 0.
                        m[j,2*i+1] = 0.
                        m[j,2*i] = 0.
                elif not self.has_spin and not self.has_eh:
                    for j in range(m.shape[0]):
                        m[j,i] = 0.
                        m[i,j] = 0.
                else: raise
        return m
    self = self.copy()
    self.modify_hamiltonian_matrices(fm)
    return self




import numpy as np

def write_magnetization(self,nrep=5):
    """Extract the magnetization and write it in a file"""
    mx = self.extract("mx")
    my = self.extract("my")
    mz = self.extract("mz")
    g = self.geometry
    g.write_profile(mx,name="MX.OUT",normal_order=True,nrep=nrep)
    g.write_profile(my,name="MY.OUT",normal_order=True,nrep=nrep)
    g.write_profile(mz,name="MZ.OUT",normal_order=True,nrep=nrep)
    # this is just a workaround
    m = np.genfromtxt("MX.OUT").transpose()
    (x,y,z,mx) = m[0],m[1],m[2],m[3]
    my = np.genfromtxt("MY.OUT").transpose()[3]
    mz = np.genfromtxt("MZ.OUT").transpose()[3]
    np.savetxt("MAGNETISM.OUT",np.array([x,y,z,mx,my,mz]).T)
    return np.array([mx,my,mz])



# this library writes down the current that would be measured by placing
# a constact in certain atoms, and putting other contact anywhere
from __future__ import print_function
from scipy.sparse import csc_matrix
from . import correlator
import numpy as np

def write_correlator(h,index=None,e=0.0,delta=0.01):
  """Write the correlator in a file"""
  if h.dimensionality!=0: raise # only for 0d
  if h.has_spin: raise # not implemented
  intra = csc_matrix(h.intra) # hamiltonian 
  d = np.zeros(intra.shape[0]) # correlator
  if True:
    ii = 0
    for i in index: # loop over atoms
      d += correlator.gij(intra,i=i,e=e,delta=delta) # add contribution
      print("Done",ii,"of",len(index))
      ii += 1
  else:
    print("Full inversion")
    g = (np.identity(intra.shape[0])*(e+delta*1j)-intra.todense()).I
    for i in index: 
      gz = np.array(g[:,i]).reshape(g.shape[0])
      d += (gz*np.conjugate(gz)).real
  g = h.geometry
  np.savetxt("CORRELATOR.OUT",np.matrix([g.x,g.y,d]).T)
  print("Writen CORRELATOR.OUT")

